@@ -6,10 +6,11 @@
 #' @param dialect The preference query dialect, which determines the Syntax of the returned query. This has to be one of the following:
 #' \describe{
 #'    \item{\code{'EXASOL'}}{Syntax of the "Skyline" feature of the commercial database EXASOL EXASolution 5.}
-#'     \item{\code{'Preference SQL'}}{Syntax of the Preference SQL system. 
+#'    \item{\code{'Preference SQL'}}{Syntax of the Preference SQL system. 
 #'      This is a research prototype developed at the Chair of Databases and Information Systems of the University of Augsburg. 
 #'      See references for details.}
 #' }
+#' @param df Optional to specifiy a dataframe on which the preference operates causing a partial evaluation. See \code{\link{show.pref}} for details.
 #'     
 #' @details 
 #' 
@@ -36,7 +37,7 @@
 #' show.query(low(a) * high(b), dialect = 'Preference SQL')
 #' 
 #' @export
-show.query <- function(p, dialect = 'EXASOL') {
+show.query <- function(p, dialect = 'EXASOL', df = NULL) {
  
   # Check dialect
   if (!(dialect %in% c(EXA, PSQL))) stop(paste0('The dialect "', dialect, '" is unknown.'))
@@ -45,7 +46,44 @@ show.query <- function(p, dialect = 'EXASOL') {
   if (is.empty.pref(p)) return("")
   
   # Return PREFERRING-clause
-  return(paste0('PREFERRING ', show_pref(p, dialect, 'PREF'))) # parent node "PREF" means root node
+  return(paste0('PREFERRING ', show_pref_sql(p, dialect, 'PREF', get_static_terms(df)))) # parent node "PREF" means root node
+}
+
+
+#' Show a (partially evaluated) preference
+#' 
+#' @param p The preference to be shown.
+#' @param df (Optional) A dataframe on which the preference operates.
+#' 
+#' @details If \code{df} is not given this function is identical to typing \code{p} on the console or calling \code{show(p)}.
+#' This standard show function does not do any evaluation and just converts the expressions to characters.
+#' A given dataframe causes that all expressions in \code{p} are evaluated except the attributes in \code{p}, 
+#' i.e. the column names in \code{df}. The content of the dataframe \code{df} does not matter; 
+#' only \code{colnames(df)} is taken to get the "free variables" in \code{p}
+#' 
+#' @examples
+#' 
+#' f <- function(x) 2*x
+#' p <- true(cyl == f(1))
+#' 
+#' # prints 'cyl == f(x)'
+#' p
+#' 
+#' # prints 'cyl == 2'
+#' show.pref(p, mtcars)
+#' 
+#' @export
+show.pref <- function(p, df = NULL) {
+
+  # Output string representation
+  return(cat(paste0('[Preference] ', unbrace(p$get_str(static_terms = get_static_terms(df))))))
+}
+
+# Get attributes of a data set as symbols
+get_static_terms <- function(df) {
+  # Get static terms (all attributes of dataset, if given)
+  if (!is.null(df)) return(lapply(colnames(df), as.symbol))
+  else return(NULL)  
 }
 
 # Available Dialects
@@ -67,25 +105,26 @@ EXASOL_PREF_MAP = list('*'        = 'PLUS',
 
 
 
-show_pref <- function(p, dialect, parent_op = '') {
+show_pref_sql <- function(p, dialect, parent_op = '', static_terms = NULL) {
   
   if (dialect == EXA) use_map <- EXASOL_PREF_MAP
   else                use_map <- PSQL_PREF_MAP
   
   if (is.basepref(p)) {
-    return(show_base_pref(p, dialect))
+    return(show_base_pref(p, dialect, static_terms))
   } else if (is.reversepref(p)) { 
     if (dialect == EXA) # INVERSE is notated as prefix in EXASOL
-      return(paste0(use_map[['reverse']], ' (', show_pref(p$p, dialect), ')'))
+      return(paste0(use_map[['reverse']], ' (', show_pref_sql(p$p, dialect, static_terms = static_terms), ')'))
     else  # "DUAL" is notated as suffix in Preference SQL!
-      return(paste0('(', show_pref(p$p, dialect), ') ', use_map[['reverse']]))
+      return(paste0('(', show_pref_sql(p$p, dialect, static_terms = static_terms), ') ', use_map[['reverse']]))
       
   } else if (is.complexpref(p)) {
     # usual complex preference (not inverse!)
     opchr <- use_map[[p$op]]
     # Check if operator is available in the given dialect
     if (is.na(opchr)) stop('Operator "', p$op, '" is not available in the dialect "', dialect, '".')
-    res <- paste0(show_pref(p$p1, dialect, opchr), ' ', opchr, ' ', show_pref(p$p2, dialect, opchr))
+    res <- paste0(show_pref_sql(p$p1, dialect, opchr, static_terms = static_terms), ' ', opchr, ' ', 
+                  show_pref_sql(p$p2, dialect, opchr, static_terms = static_terms))
     if (parent_op != opchr && parent_op != 'PREF') res <- paste0('(', res, ')')
     return(res)
   }
@@ -93,31 +132,34 @@ show_pref <- function(p, dialect, parent_op = '') {
 
 
 # Translate Preference to PrefSQL
-show_base_pref <- function(p, dialect) {
+show_base_pref <- function(p, dialect, static_terms = NULL) {
+  
+  expr_str <- get_expr_sql(p, static_terms)
+  
   if (dialect == EXA) {
     if (is.lowpref(p)) {
-      return(paste0('LOW ', get_expr(p)))
+      return(paste0('LOW ', expr_str))
     } else if (is.highpref(p)) {
-      return(paste0('HIGH ', get_expr(p)))
+      return(paste0('HIGH ', expr_str))
     } else if (is.truepref(p)) {
-      return(paste0(get_expr(p)))
+      return(paste0(expr_str))
     }
   } else {
     if (is.lowpref(p)) {
-      return(paste0(get_expr(p), ' LOWEST'))
+      return(paste0(expr_str, ' LOWEST'))
     } else if (is.highpref(p)) {
-      return(paste0(get_expr(p), ' HIGHEST'))
+      return(paste0(expr_str, ' HIGHEST'))
     } else if (is.truepref(p)) {
-      return(paste0(get_expr(p), ' = TRUE'))
+      return(paste0(expr_str, ' = TRUE'))
     }
   }
 }
 
 
-# Get expression of base preference
-get_expr <- function(p) {
-  expr <- p$expr
-  if (length(expr[[1]]) == 1) return(as.character(p$expr))
-  else return(paste0('(', as.character(p$expr), ')')) # embrace complex expressions
+# Get (evaluated, if static_terms is given) expression of base preference
+# Add braces for non-single term 
+get_expr_sql <- function(p, static_terms = NULL) {
+  expr_str <- p$get_expr_str(static_terms)
+  if (length(p$expr[[1]]) != 1) return(paste0('(', expr_str, ')'))  # embrace complex expressions
+  else return(expr_str) 
 }
-
